# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 CERN.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio administration domains view module."""

from invenio_administration.views.base import (
    AdminResourceCreateView,
    AdminResourceDetailView,
    AdminResourceEditView,
    AdminResourceListView,
)
from invenio_i18n import lazy_gettext as _


class DomainAdminMixin:
    """Common admin properties."""

    resource_config = "domains_resource"
    extension_name = "invenio-users-resources"

    api_endpoint = "/domains"
    pid_path = "domain"

    create_view_name = "domains_create"
    list_view_name = "domains"

    display_search = True
    display_delete = True
    display_create = True
    display_edit = True


class DomainsListView(DomainAdminMixin, AdminResourceListView):
    """Search admin view."""

    name = "domains"
    title = _("Domains")
    menu_label = _("Domains")
    category = _("Site management")
    icon = "world"

    item_field_list = {
        "domain": {"text": _("Domain"), "order": 1, "width": 3},
        "tld": {"text": _("TLD"), "order": 2, "width": 1},
        "status_name": {"text": _("Status"), "order": 3, "width": 1},
        "num_users": {"text": _("Users"), "order": 6, "width": 1},
        "num_active": {"text": _("Active"), "order": 7, "width": 1},
        "num_inactive": {"text": _("Inactive"), "order": 8, "width": 1},
        "num_confirmed": {"text": _("Confirmed"), "order": 9, "width": 1},
        "num_verified": {"text": _("Verified"), "order": 10, "width": 1},
        "num_blocked": {"text": _("Blocked"), "order": 11, "width": 1},
        "links": {"text": "", "order": 13, "width": 1},
    }

    search_config_name = "USERS_RESOURCES_DOMAINS_SEARCH"
    search_sort_config_name = "USERS_RESOURCES_DOMAINS_SORT_OPTIONS"
    search_facets_config_name = "USERS_RESOURCES_DOMAINS_SEARCH_FACETS"

    template = "invenio_app_rdm/administration/domains_search.html"

    search_request_headers = {"Accept": "application/vnd.inveniordm.v1+json"}


class FormMixin:
    """Mixin class for form fields."""

    form_fields = {
        "domain": {
            "order": 1,
            "text": _("Domain"),
            "description": _("Domain name (all lowercase)"),
        },
        "status_name": {
            "order": 2,
            "required": True,
            "text": _("Status"),
            "description": _(
                "Status of the domain. One of new (domain needs review), moderated (users in domain require moderation), verified (users in domain does not require moderation) or blocked (users cannot register using this domain)."
            ),
            "options": [
                {"title_l10n": "New", "id": "new"},
                {"title_l10n": "Moderated", "id": "moderated"},
                {"title_l10n": "Verified", "id": "verified"},
                {"title_l10n": "Blocked", "id": "blocked"},
            ],
            "placeholder": "Select a status",
        },
        "category_name": {
            "order": 3,
            "text": _("Category"),
            "description": _("A label to categorise the domain."),
            "options": [
                # TODO: Needs to be dynmaically loaded from domain category table.
                {"title_l10n": "Organization", "id": "organization"},
                {"title_l10n": "Company", "id": "company"},
                {"title_l10n": "Mail provider", "id": "mailprovider"},
                {"title_l10n": "Spammer", "id": "spammer"},
            ],
            "placeholder": "Select a category",
        },
        "flagged": {
            "order": 4,
            "text": _("Flagged"),
            "description": _(
                "Used by automatic processes to flag the domain as requiring review."
            ),
        },
        "flagged_source": {
            "order": 5,
            "text": _("Source of flag"),
            "description": _("Which source flagged the domain."),
        },
    }


class DomainsCreateView(DomainAdminMixin, FormMixin, AdminResourceCreateView):
    """Configuration for Banner create view."""

    name = "domains_create"
    url = "/domains/create"
    title = _("Create domain")


class DomainsEditView(DomainAdminMixin, FormMixin, AdminResourceEditView):
    """Domain edit view."""

    name = "domains_edit"
    url = "/domains/<pid_value>/edit"
    title = _("Edit domain")


class DomainsDetailView(DomainAdminMixin, AdminResourceDetailView):
    """Domain detail view."""

    url = "/domains/<pid_value>"
    name = "domains_details"
    title = _("Domain details")

    item_field_list = {
        "domain": {"text": _("Domain"), "order": 1},
        "tld": {"text": _("Top-level domain"), "order": 2},
        "status_name": {"text": _("Status"), "order": 3},
        "category_name": {"text": _("Category"), "order": 4},
        "flagged": {"text": _("Flagged"), "order": 5},
        "flagged_source": {"text": _("Source of flag"), "order": 6},
        "created": {"text": _("Created"), "order": 7},
        "updated": {"text": _("Updated"), "order": 8},
        "org": {"text": _("Organisation"), "order": 9},
        "num_users": {"text": _("Users"), "order": 10},
        "num_active": {"text": _("Active"), "order": 11},
        "num_inactive": {"text": _("Inactive"), "order": 12},
        "num_confirmed": {"text": _("Confirmed"), "order": 13},
        "num_verified": {"text": _("Verified"), "order": 14},
        "num_blocked": {"text": _("Blocked"), "order": 15},
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2024 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
# Copyright (C) 2023-2024 Graz University of Technology.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Communities UI blueprints module."""

from flask import Blueprint, current_app, render_template, request
from flask_login import current_user
from invenio_communities.communities.resources.serializer import (
    UICommunityJSONSerializer,
)
from invenio_communities.errors import CommunityDeletedError
from invenio_communities.views.ui import (
    not_found_error,
    record_permission_denied_error,
    record_tombstone_error,
)
from invenio_i18n import lazy_gettext as _
from invenio_pidstore.errors import PIDDeletedError, PIDDoesNotExistError
from invenio_rdm_records.collections import search_app_context as c_search_app_context
from invenio_records_resources.services.errors import (
    PermissionDeniedError,
    RecordPermissionDeniedError,
)

from ..searchapp import search_app_context
from .communities import (
    communities_browse,
    communities_detail,
    communities_home,
    community_collection,
    community_static_page,
)


def _show_browse_page():
    """Whether the browse page should be visible in the menu."""
    return (
        current_app.config.get("COMMUNITIES_SHOW_BROWSE_MENU_ENTRY", False)
        and request.community["children"]["allow"]
    )


def create_ui_blueprint(app):
    """Register blueprint routes on app."""
    routes = app.config["RDM_COMMUNITIES_ROUTES"]

    blueprint = Blueprint(
        "invenio_app_rdm_communities",
        __name__,
        template_folder="../templates",
        static_folder="../static",
    )

    blueprint.add_url_rule(
        routes["community-detail"],
        view_func=communities_detail,
        strict_slashes=False,
    )

    blueprint.add_url_rule(
        routes["community-home"],
        view_func=communities_home,
    )

    blueprint.add_url_rule(
        routes["community-browse"],
        view_func=communities_browse,
    )

    blueprint.add_url_rule(
        routes["community-static-page"],
        view_func=community_static_page,
    )

    blueprint.add_url_rule(
        routes["community-collection"],
        view_func=community_collection,
    )
    # Register error handlers
    blueprint.register_error_handler(
        PermissionDeniedError, record_permission_denied_error
    )
    blueprint.register_error_handler(
        RecordPermissionDeniedError, record_permission_denied_error
    )
    blueprint.register_error_handler(PIDDeletedError, record_tombstone_error)
    blueprint.register_error_handler(CommunityDeletedError, record_tombstone_error)
    blueprint.register_error_handler(PIDDoesNotExistError, not_found_error)
    blueprint.register_error_handler(PIDDoesNotExistError, not_found_error)

    # Register context processor
    blueprint.app_context_processor(search_app_context)
    blueprint.app_context_processor(c_search_app_context)

    return blueprint



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2021 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2021 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Views related to records and deposits."""

from flask import Blueprint
from invenio_drafts_resources.resources.records.errors import DraftNotCreatedError
from invenio_pidstore.errors import (
    PIDDeletedError,
    PIDDoesNotExistError,
    PIDUnregistered,
)
from invenio_rdm_records.services.errors import RecordDeletedException
from invenio_records_resources.services.errors import (
    FileKeyNotFoundError,
    PermissionDeniedError,
    RecordPermissionDeniedError,
)

from ...theme.views import create_url_rule
from ..searchapp import search_app_context
from .deposits import community_upload, deposit_create, deposit_edit
from .filters import (
    can_list_files,
    compact_number,
    custom_fields_search,
    get_scheme_label,
    has_images,
    has_previewable_files,
    localize_number,
    make_files_preview_compatible,
    namespace_url,
    order_entries,
    pid_url,
    select_preview_file,
    to_previewer_files,
    transform_record,
    truncate_number,
)
from .records import (
    draft_not_found_error,
    not_found_error,
    record_detail,
    record_export,
    record_file_download,
    record_file_preview,
    record_from_pid,
    record_latest,
    record_media_file_download,
    record_permission_denied_error,
    record_thumbnail,
    record_tombstone_error,
)


#
# Registration
#
def create_blueprint(app):
    """Register blueprint routes on app."""
    routes = app.config.get("APP_RDM_ROUTES")
    communities_routes = app.config.get("COMMUNITIES_ROUTES")

    blueprint = Blueprint(
        "invenio_app_rdm_records",
        __name__,
        template_folder="../templates",
    )

    # Record URL rules
    blueprint.add_url_rule(
        **create_url_rule(
            routes["record_detail"],
            default_view_func=record_detail,
        )
    )

    blueprint.add_url_rule(
        **create_url_rule(
            routes["record_latest"],
            default_view_func=record_latest,
        )
    )

    rdm_records_ext = app.extensions["invenio-rdm-records"]
    schemes = rdm_records_ext.records_service.config.pids_providers.keys()
    schemes = ",".join(schemes)
    if schemes:
        blueprint.add_url_rule(
            **create_url_rule(
                routes["record_from_pid"].format(schemes=schemes),
                default_view_func=record_from_pid,
            )
        )

    blueprint.add_url_rule(
        **create_url_rule(
            routes["record_export"],
            default_view_func=record_export,
        )
    )

    blueprint.add_url_rule(
        **create_url_rule(
            routes["record_file_preview"],
            default_view_func=record_file_preview,
        )
    )

    blueprint.add_url_rule(
        **create_url_rule(
            routes["record_file_download"],
            default_view_func=record_file_download,
        )
    )
    blueprint.add_url_rule(
        **create_url_rule(
            routes["record_thumbnail"],
            default_view_func=record_thumbnail,
        )
    )

    blueprint.add_url_rule(
        **create_url_rule(
            routes["record_media_file_download"],
            default_view_func=record_media_file_download,
        )
    )

    blueprint.add_url_rule(
        **create_url_rule(
            routes["deposit_create"],
            default_view_func=deposit_create,
        )
    )

    blueprint.add_url_rule(
        **create_url_rule(
            routes["deposit_edit"],
            default_view_func=deposit_edit,
        )
    )

    blueprint.add_url_rule(
        **create_url_rule(
            communities_routes["upload"],
            default_view_func=community_upload,
        )
    )

    # Register error handlers
    blueprint.register_error_handler(PIDDeletedError, record_tombstone_error)
    blueprint.register_error_handler(PIDDoesNotExistError, not_found_error)
    blueprint.register_error_handler(PIDUnregistered, not_found_error)
    blueprint.register_error_handler(KeyError, not_found_error)
    blueprint.register_error_handler(FileKeyNotFoundError, not_found_error)
    blueprint.register_error_handler(DraftNotCreatedError, draft_not_found_error)
    blueprint.register_error_handler(
        PermissionDeniedError, record_permission_denied_error
    )
    blueprint.register_error_handler(
        RecordPermissionDeniedError, record_permission_denied_error
    )
    blueprint.register_error_handler(RecordDeletedException, record_tombstone_error)

    # Register template filters
    blueprint.add_app_template_filter(can_list_files)
    blueprint.add_app_template_filter(make_files_preview_compatible)
    blueprint.add_app_template_filter(pid_url)
    blueprint.add_app_template_filter(select_preview_file)
    blueprint.add_app_template_filter(to_previewer_files)
    blueprint.add_app_template_filter(has_previewable_files)
    blueprint.add_app_template_filter(order_entries)
    blueprint.add_app_template_filter(get_scheme_label)
    blueprint.add_app_template_filter(has_images)
    blueprint.add_app_template_filter(localize_number)
    blueprint.add_app_template_filter(compact_number)
    blueprint.add_app_template_filter(truncate_number)
    blueprint.add_app_template_filter(namespace_url)
    blueprint.add_app_template_filter(custom_fields_search)
    blueprint.add_app_template_filter(transform_record)

    # Register context processor
    blueprint.app_context_processor(search_app_context)

    return blueprint



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2024 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
# Copyright (C)      2022 Graz University of Technology.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""JS/CSS Webpack bundles for theme."""

from invenio_assets.webpack import WebpackThemeBundle

theme = WebpackThemeBundle(
    __name__,
    "assets",
    default="semantic-ui",
    themes={
        "semantic-ui": dict(
            entry={
                "invenio-app-rdm-landing-page": "./js/invenio_app_rdm/landing_page/index.js",
                "invenio-app-rdm-landing-page-access-form": "./js/invenio_app_rdm/landing_page/access.js",
                "invenio-app-rdm-landing-page-theme": "./js/invenio_app_rdm/landing_page/theme.js",
                "invenio-app-rdm-deposit": "./js/invenio_app_rdm/deposit/index.js",
                "invenio-app-rdm-search": "./js/invenio_app_rdm/search/index.js",
                "invenio-app-rdm-user-uploads": "./js/invenio_app_rdm/user_dashboard/uploads.js",
                "invenio-app-rdm-user-communities": "./js/invenio_app_rdm/user_dashboard/communities.js",
                "invenio-app-rdm-user-requests": "./js/invenio_app_rdm/user_dashboard/requests.js",
                "invenio-app-rdm-community-records-search": "./js/invenio_app_rdm/communityRecordsSearch/index.js",
                "invenio-app-rdm-collections-search": "./js/invenio_app_rdm/collectionRecordsSearch/index.js",
                "base-theme-rdm": "./js/invenio_app_rdm/theme.js",
                "iiif-simple-previewer": "./less/invenio_app_rdm/previewer/iiif_simple.less",
                "invenio-app-rdm-frontpage": "./js/invenio_app_rdm/frontpage/index.js",
                "overridable-registry": "./js/invenio_app_rdm/overridableRegistry/index.js",
                "invenio-users-moderation-administration": "./js/invenio_app_rdm/administration/userModeration/index.js",
                "invenio-users-administration": "./js/invenio_app_rdm/administration/users/index.js",
                "invenio-records-administration": "./js/invenio_app_rdm/administration/records/index.js",
                "invenio-drafts-administration": "./js/invenio_app_rdm/administration/drafts/index.js",
                "invenio-domains-administration": "./js/invenio_app_rdm/administration/domains/index.js",
                "invenio-communities-browse": "./js/invenio_app_rdm/subcommunity/browse.js",
            },
            dependencies={
                "@babel/runtime": "^7.9.0",
                "@tinymce/tinymce-react": "^4.3.0",
                "tinymce": "^6.7.2",
                "formik": "^2.1.0",
                "i18next": "^20.3.0",
                "i18next-browser-languagedetector": "^6.1.0",
                "luxon": "^1.23.0",
                "path": "^0.12.7",
                "prop-types": "^15.7.2",
                "react-copy-to-clipboard": "^5.0.0",
                "react-dnd": "^11.1.0",
                "react-dnd-html5-backend": "^11.1.0",
                "react-dropzone": "^11.0.0",
                "react-i18next": "^11.11.0",
                "react-invenio-forms": "^4.0.0",
                "react-searchkit": "^3.0.0",
                "yup": "^0.32.0",
                "json-diff-kit": "^1.0.30",
            },
            aliases={
                # Define Semantic-UI theme configuration needed by
                # Invenio-Theme in order to build Semantic UI (in theme.js
                # entry point). theme.config itself is provided by
                # cookiecutter-invenio-rdm.
                "../../theme.config$": "less/theme.config",
                "../../less/site": "less/site",
                "../../less": "less",
                "themes/rdm": "less/invenio_app_rdm/theme",
                "@less/invenio_app_rdm": "less/invenio_app_rdm",
                "@js/invenio_app_rdm": "js/invenio_app_rdm",
                "@translations/invenio_app_rdm": "translations/invenio_app_rdm",
            },
            copy=[
                # Copy some assets into "static/dist", as TinyMCE requires that
                # Note that the base path for all entries is the `config.json` directory
                {
                    "from": "../node_modules/tinymce/skins/content/default/content.css",
                    "to": "../../static/dist/js/skins/content/default",
                },
                {
                    "from": "../node_modules/tinymce/skins/ui/oxide/skin.min.css",
                    "to": "../../static/dist/js/skins/ui/oxide",
                },
                {
                    "from": "../node_modules/tinymce/skins/ui/oxide/content.min.css",
                    "to": "../../static/dist/js/skins/ui/oxide",
                },
                {
                    "from": "../node_modules/json-diff-kit/dist/viewer.css",
                    "to": "../../static/css/json-diff-kit.css",
                },
            ],
        ),
    },
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Northwestern University.
#
# Invenio-App-RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test Signposting.

See https://signposting.org/FAIR/#level2 for more information on Signposting
"""


def test_link_in_landing_page_response_headers(running_app, client, record):
    res = client.head(f"/records/{record.id}")

    assert (
        res.headers["Link"]
        == f'<https://127.0.0.1:5000/api/records/{record.id}> ; rel="linkset" ; type="application/linkset+json"'  # noqa
    )


def test_link_in_content_resource_response_headers(
    running_app, client, record_with_file
):
    filename = "article.txt"

    res = client.head(f"/records/{record_with_file.id}/files/{filename}")

    assert (
        res.headers["Link"]
        == f'<https://127.0.0.1:5000/api/records/{record_with_file.id}> ; rel="linkset" ; type="application/linkset+json"'  # noqa
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2025 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio App RDM."""

# Note, that dev versions must have an extra dot compared to alpha/beta/rc
# versions in the version string. Examples:
#
# Correct: "1.0.0.dev0" and "1.0.0a1" and "1.0.0rc1"
# Incorrect: "1.0.0dev0" and "1.0.0.a1" and "1.0.0.rc1"
#
# See PEP 0440 for details - https://www.python.org/dev/peps/pep-0440

__version__ = "13.0.0b2.dev2"

__all__ = ("__version__",)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-App-RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Record migration script from InvenioRDM 8.0 to 9.0.

Disclaimer: This script is intended to be executed *only once*, namely when
upgrading from InvenioRDM 8.0 to 9.0!
If this script is executed at any other time, probably the best case scenario
is that nothing happens!
"""

from click import echo, secho
from invenio_db import db
from invenio_rdm_records.records.api import RDMRecord


def execute_upgrade():
    """Execute the upgrade from InvenioRDM 8.0 to 9.0.

    Please read the disclaimer on this module before thinking about executing
    this function!
    """

    def update_funding_field(record):
        try:
            echo(f"Updating record: {record['id']}... ", nl=False)
            for funding in record.metadata.get("funding", []):
                award = funding.get("award", {})
                funder = funding.get("funder", {})
                if award.get("identifier") and award.get("scheme"):
                    award_identifier = award.pop("identifier")
                    award_scheme = award.pop("scheme")
                    funding["award"]["identifiers"] = [
                        {"identifier": award_identifier, "scheme": award_scheme}
                    ]
                if funder.get("identifier") and funder.get("scheme"):
                    funder.pop("identifier")
                    funder.pop("scheme")
                if award.get("title", ""):
                    award_title = award.pop("title")
                    funding["award"]["title"] = {"en": award_title}
                if funder.get("title", ""):
                    funder.pop("title")
            secho("OK", fg="green")
            return record
        except Exception as e:
            secho("Error {}".format(repr(e)), fg="red")
            return None

    errors = []

    records = RDMRecord.model_cls.query.all()
    for record in records:
        r = RDMRecord(record.data, model=record)
        r["$schema"] = "local://records/record-v5.0.0.json"
        res = update_funding_field(r)
        if res is None:
            errors.append("Record {} failed to update funding".format(r.id))
        else:
            r = res
            r.commit()

    success = not errors

    if success:
        echo("Committing to DB", nl=True)
        db.session.commit()
        secho(
            "Data migration completed, please rebuild the search indices now.",
            fg="green",
        )

    else:
        echo("Rollback", nl=True)
        db.session.rollback()
        secho(
            "Upgrade aborted due to the following errors:",
            fg="red",
            err=True,
        )

        for error in errors:
            secho(error, fg="red", err=True)

        msg = (
            "The changes have been rolled back. "
            "Please fix the above listed errors and try the upgrade again",
        )
        secho(msg, fg="yellow", err=True)


# if the script is executed on its own, perform the upgrade
if __name__ == "__main__":
    execute_upgrade()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 CERN.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio administration module for user resources."""

from .users import UsersDetailView, UsersListView

__all__ = ("UsersDetailView", "UsersListView")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023-2024 CERN.
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio-App-RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Record migration script from InvenioRDM 11.0 to 12.0.

Disclaimer: This script is intended to be executed *only once*, namely when
upgrading from InvenioRDM 11.0 to 12.0!
If this script is executed at any other time, probably the best case scenario
is that nothing happens!


This script has been tested with following data:

- record
    - internal_notes
"""

import sys

from click import secho
from invenio_db import db
from invenio_rdm_records.records.api import RDMDraft, RDMRecord


def execute_upgrade():
    """Execute the upgrade from InvenioRDM 12.0 to 13.0.0.

    Please read the disclaimer on this module before thinking about executing
    this function!
    THIS MODULE IS WORK IN PROGRESS, UNTIL official v13 release
    """

    def update_record(record):
        # skipping deleted records because can't be committed
        if record.is_deleted:
            return

        try:
            secho(f"Updating record : {record.pid.pid_value}", fg="yellow")

            # TODO: Add any record datamodel migration code here
            record.commit()

            secho(f"> Updated parent: {record.parent.pid.pid_value}", fg="green")
            secho(f"> Updated record: {record.pid.pid_value}\n", fg="green")
            return None
        except Exception as e:
            secho(f"> Error {repr(e)}", fg="red")
            error = f"Record {record.pid.pid_value} failed to update"
            return error

    secho("Starting data migration...", fg="green")

    # Migrating records and drafts
    errors = []
    for record_metadata in RDMRecord.model_cls.query.all():
        record = RDMRecord(record_metadata.data, model=record_metadata)
        error = update_record(record)

        if error:
            errors.append(error)

    for draft_metadata in RDMDraft.model_cls.query.all():
        draft = RDMDraft(draft_metadata.data, model=draft_metadata)
        error = update_record(draft)
        if error:
            errors.append(error)

    success = not errors

    if success:
        secho("Commiting to DB", nl=True)
        db.session.commit()
        secho(
            "Data migration completed, please rebuild the search indices now.",
            fg="green",
        )

    else:
        secho("Rollback", nl=True)
        db.session.rollback()
        secho(
            "Upgrade aborted due to the following errors:",
            fg="red",
            err=True,
        )

        for error in errors:
            secho(error, fg="red", err=True)

        msg = (
            "The changes have been rolled back. "
            "Please fix the above listed errors and try the upgrade again",
        )
        secho(msg, fg="yellow", err=True)

        sys.exit(1)


# if the script is executed on its own, perform the upgrade
if __name__ == "__main__":
    execute_upgrade()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Test fixtures for application vocabulary fixtures."""

import pytest
from invenio_app.factory import create_api


@pytest.fixture(scope="module")
def create_app():
    """Application factory fixture."""
    return create_api


@pytest.fixture(scope="module")
def cli_runner(base_app):
    """Create a CLI runner for testing a CLI command."""

    def cli_invoke(command, *args, input=None):
        return base_app.test_cli_runner().invoke(command, args, input=input)

    return cli_invoke



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Northwestern University.
#
# This file is lifted from Invenio-RDM-Records .
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Convenient URL generation.

InvenioRDM poses challenges to url generation that Flask's url_for cannot handle out
of the gate.

- InvenioRDM is actually 2 applications mounted on different url_prefixes:
  `url_for` in the API application isn't aware of the UI application endpoints
- The endpoint names are relatively hidden / spread out and APP_RDM_ROUTES is usually
  the interface to name them.
- Need to be able to generate urls outside of request context without thinking about it,

This module contains minimal methods to generate URLs correctly without much
engineering. Over time, it can be made more abstract, complex and powerful and even
extracted into its own package to solve url generation across InvenioRDM once and for
all.

Design decisions:

- Generated urls are absolute for now
"""

from flask import current_app


def record_url_for(_app="ui", pid_value=""):
    """Return url for record route."""
    assert _app in ["ui", "api"]

    site_app = _app.upper()
    url_prefix = current_app.config.get(f"SITE_{site_app}_URL", "")

    # We use [] so that this fails and brings to attention the configuration
    # problem if APP_RDM_ROUTES.record_detail is missing
    url_path = current_app.config["APP_RDM_ROUTES"]["record_detail"].replace(
        "<pid_value>", pid_value
    )

    return "/".join(p.strip("/") for p in [url_prefix, url_path])


def download_url_for(pid_value="", filename=""):
    """Return url for download route."""
    url_prefix = current_app.config.get(f"SITE_UI_URL", "")

    # We use [] so that this fails and brings to attention the configuration
    # problem if APP_RDM_ROUTES.record_file_download is missing
    url_path = (
        current_app.config["APP_RDM_ROUTES"]["record_file_download"]
        .replace("<pid_value>", pid_value)
        .replace("<path:filename>", filename)
    )

    return "/".join(p.strip("/") for p in [url_prefix, url_path])
